#include "mainwindow.h"
#include "./ui_mainwindow.h"
#include <QDebug>
#include <memory>
#include <qnamespace.h>

#define INIT_SPAWN_SPEED 1500.0
#define SPAWN_SPEED_INCREASE 100.0
#define MIN_SPAWN_SPEED 100.0

MainWindow::MainWindow(QWidget* parent) : QMainWindow(parent), ui(new Ui::MainWindow), leftArrowPressed{false}, rightArrowPressed{false}, m_spawnInterval{INIT_SPAWN_SPEED} {
    ui->setupUi(this);

    connect(&m_spawnTimer, &QTimer::timeout, this, &MainWindow::onBallSpawn);
    m_spawnTimer.start(m_spawnInterval);

    platform = std::make_shared<Platform>(this);
    spawnBall();
}

MainWindow::~MainWindow() {
    delete ui;
}

void MainWindow::paintEvent(QPaintEvent*) {
    QPainter painter(this);

    painter.fillRect(rect(), Qt::darkGray);

    painter.setBrush(QBrush(Qt::white));
    painter.setPen(Qt::black);

    for (auto ball : balls) {
        ball->draw(painter);
    }

    platform->draw(painter);
}

void MainWindow::deleteBall(Ball* ball) {
    // Remove the ball from the list by checking the raw pointer
    // This will also free it (smart ptrs).
    for (auto it = balls.begin(); it != balls.end(); ++it) {
        if (it->get() == ball) {
            qDebug() << "Deleting ball";
            balls.erase(it);
            break;
        }
    }
}

void MainWindow::spawnBall() {
    auto ball = std::make_shared<Ball>(this, platform);
    connect(ball.get(), &Ball::fellThrough, this, &MainWindow::ballFell);
    connect(ball.get(), &Ball::caught, this, &MainWindow::ballCaught);
    balls.append(ball);
}

void MainWindow::ballFell(Ball* ball) {
    deleteBall(ball);
    // TODO: Game over
}

void MainWindow::ballCaught(Ball* ball) {
    m_spawnInterval -= SPAWN_SPEED_INCREASE;
    if (m_spawnInterval < MIN_SPAWN_SPEED)
        m_spawnInterval = MIN_SPAWN_SPEED;
    deleteBall(ball);
}

void MainWindow::onBallSpawn() {
    spawnBall();
    m_spawnTimer.start(m_spawnInterval);
}

void MainWindow::keyPressEvent(QKeyEvent* event) {
    switch (event->key()) {
        case Qt::Key_Left: leftArrowPressed = true; break;
        case Qt::Key_Right: rightArrowPressed = true; break;
    }
}

void MainWindow::keyReleaseEvent(QKeyEvent* event) {
    switch (event->key()) {
        case Qt::Key_Left: leftArrowPressed = false; break;
        case Qt::Key_Right: rightArrowPressed = false; break;
    }
}
